﻿using Infragistics.Win;
using Infragistics.Win.UltraWinMaskedEdit;
using Infragistics.Win.UltraWinToolbars;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ToolbarsManagerPopupDrawFilterTest
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
            ultraToolbarsManager1.DrawFilter = new ToolbarsManagerDrawFilter();

            this.Load += Form1_Load;
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            UltraToolbar toolbar = ultraToolbarsManager1.Toolbars[0];
            PopupMenuTool menu = toolbar.Tools[0] as PopupMenuTool;

            foreach (LabelTool tool in menu.Tools)
            {
                tool.SharedProps.DescriptionOnMenu = "Description on Menu!";
            }
        }
    }


    public class ToolbarsManagerDrawFilter : IUIElementDrawFilter
    {
        //private string m_text;
        //private MaskedEditTool m_editor;
        //private ImageList m_imageList;
        //private UltraToolbarsManager m_toolbarsManager;        

        private const int ArrowAreaWidth = 30;

        private PopupMenuTool m_quickStartPopupMenuTool;
        public PopupMenuTool QuickStartPopupMenuTool
        {
            get => m_quickStartPopupMenuTool;
            set => m_quickStartPopupMenuTool = value;
        }

        //public ToolbarsManagerDrawFilter(string text, MaskedEditTool maskedEditTool, ImageList imageList)
        //{
        //    m_text = text;
        //    m_editor = maskedEditTool;
        //    m_toolbarsManager = maskedEditTool.ToolbarsManager;
        //    m_imageList = imageList;            
        //}

        DrawPhase IUIElementDrawFilter.GetPhasesToFilter(ref UIElementDrawParams drawParams)
        {
            return DrawPhase.BeforeDrawForeground | DrawPhase.AfterDrawElement;
        }

        bool IUIElementDrawFilter.DrawElement(DrawPhase drawPhase, ref UIElementDrawParams drawParams)
        {
            System.Diagnostics.Debug.WriteLine(drawParams.Element);

            if (drawParams.Element is PopupMenuItemDescriptionUIElement descriptionUIElement &&
                descriptionUIElement.Value.GetText(true).StartsWith("Description") &&
                drawPhase == DrawPhase.AfterDrawElement)
            {                

                if (descriptionUIElement.Control.Parent is Form parentForm && parentForm.Text.Contains("HomeButton")) return false;

                UIElement parent = descriptionUIElement.Parent;

                //SdnPMItem mandantItem = GetMandantItem(parent);

                //bool isFavorite = !mandantItem["IsQuickstartFavorite"].IsNullOrEmpty;

                IEnumerable<ImageAndTextButtonUIElement> buttonChildElements = parent.ChildElements.OfType<ImageAndTextButtonUIElement>().ToList();
                if (!buttonChildElements.Any())
                {
                    ImageAndTextButtonUIElement imageAndTextButtonUIElement = new ImageAndTextButtonUIElement(descriptionUIElement.Parent)
                    {
                        Rect = new Rectangle(parent.RectInsideBorders.Width - ArrowAreaWidth, parent.RectInsideBorders.Top, 16, 16),
                        Style = UIElementButtonStyle.Borderless
                    };

                    AddImageAndTooltipp(imageAndTextButtonUIElement);

                    imageAndTextButtonUIElement.Appearance = new Infragistics.Win.Appearance();
                    imageAndTextButtonUIElement.Appearance.BackColor = Color.Red;                             
                    imageAndTextButtonUIElement.ElementClick += ImageAndTextButtonUIElementElementClick;
                    parent.ChildElements.Add(imageAndTextButtonUIElement);
                }
                else if (buttonChildElements.Count() == 1)
                {
                    ImageAndTextButtonUIElement imageAndTextButtonUIElement = buttonChildElements.First();
                    AddImageAndTooltipp(imageAndTextButtonUIElement);
                }

                return true;
            }
            if (drawPhase == DrawPhase.BeforeDrawForeground)
            {
                UIElement parent = null;

                if (drawParams.Element is TextUIElement uiElement &&
                    uiElement.Parent is MaskedEditUIElement)
                {
                    parent = uiElement.Parent;
                }
                //else if (drawParams.Element is MaskedEditUIElement maskedEditUIElementFocused &&
                //         !(m_editor is null) && !(m_editor.Value is null) && string.IsNullOrEmpty(m_editor.Value.ToString()))
                //{
                //    parent = maskedEditUIElementFocused.Parent;
                //}

                while (!(parent is null))
                {
                    if (parent is PopupMenuItemUIElement)
                    {
                        break;
                    }

                    parent = parent.Parent;
                }

                if (!(parent is null))
                {
                    Rectangle drawingRect = drawParams.ElementDrawingRect;
                    Point centerPoint = new Point(drawingRect.X + drawingRect.Width / 2, drawingRect.Y + drawingRect.Height / 2);
                    //SizeF textSize = drawParams.Graphics.MeasureString(m_text, drawParams.Font);
                    Brush textBrush = new SolidBrush(Color.DarkGray);
                    //drawParams.Graphics.DrawString(m_text, drawParams.Font, textBrush, new PointF(drawingRect.X + 2, centerPoint.Y - textSize.Height / 2));
                    return true;
                }

                return false;
            }

            return false;
        }

        private void AddImageAndTooltipp(ImageAndTextButtonUIElement imageAndTextButtonUIElement)
        {                        
            imageAndTextButtonUIElement.Image = Bitmap.FromFile("../../Images/RedX.png");
        }

        private void ImageAndTextButtonUIElementElementClick(object sender, UIElementEventArgs e)
        {
            if (e.Element is ImageAndTextButtonUIElement buttonUIElement)
            {
                PopupMenuTool quickStartPopupMenuTool = QuickStartPopupMenuTool;

                //SdnPMItem mandantItem = GetMandantItem(buttonUIElement.Parent);
                //if (!(mandantItem is null))
                //{
                //    if (mandantItem["IsQuickstartFavorite"].IsNullOrEmpty)
                //    {
                //        mandantItem["IsQuickstartFavorite"] = true;
                //    }
                //    else
                //    {
                //        mandantItem["IsQuickstartFavorite"] = false;
                //    }

                //    mandantItem["LastUsed"] = DateTime.Now;
                //}

                //if (m_mandantSelector.HasMethod("PostacceptEditors")) m_mandantSelector.CallMethodWithCircumventAccessMode(m_mandantSelector, "PostacceptEditors");

                //MaintenanceUseCaseView maintenanceUseCaseView = m_toolbarsManager.DockWithinContainer as MaintenanceUseCaseView;

                //if (!(maintenanceUseCaseView is null))
                //{
                //    maintenanceUseCaseView.FreezePainting = true;
                //}

                quickStartPopupMenuTool.ClosePopup();
                quickStartPopupMenuTool.DropDown();

                //if (!(maintenanceUseCaseView is null))
                //{
                //    maintenanceUseCaseView.FreezePainting = false;
                //}
            }
        }
    }
}
